# pi_abtest

### ab测试

- 配置定义

```rs

// ABTest配置表
#[primary=name,db=logfile,dbMonitor=true]
struct ABTestCfg1DB {
    name: String,         // 组名
    #[default={}]
    cfg:Json,    // 新增配置是否影响旧用户
}



// ABTest版本管理表
#[primary=key,db=logfile,dbMonitor=true]
struct ABTestTagVersion1DB {
    key: String,   // 固定值 version
    version:u32,      // 版本,每次清除tags的时候版本号递增
}


// ABTest表
#[primary=key,db=logfile,dbMonitor=true]
struct ABTest1DB {
    key: String,         // uid|rid
    tagVersion:u32,        // 标签版本
    #[default=new Map()]
    value: HashMap<String, String>,        // Map<abTest_name,tag>
}


```

```ts
export interface IAbTestCfg {
    name: string;                // 组名
    affectOldUser: boolean;      // 新增配置是否影响旧用户
    createStartTime: string;     // 创建tag的开始时间,形如2024-01-08 00:00:00
    createEndTime: string;       // 创建tag的结束时间,形如2024-01-08 00:00:00
    effectStartTime: string;     // tag生效的开始时间,形如2024-01-08 00:00:00
    effectEndTime: string;       // tag生效的结束时间,形如2024-01-08 00:00:00
    os: string[];                // 系统
    channel: string[];           // 渠道
    groups: IAbTestGroup[];      // 组别
    fixedTag?: string;            // 固定tag,用于ab测试完成后选中一个tag
}


interface IAbTestGroup {
    tag: string;      // 标签
    percent: number;      // 百分比
}
```




#### 运营端使用接口例子(http请求,目前运营人员可以使用postman发送,后续考虑集成到管理端):

##### 设置ABTest配置```/port/setAbTestCfg```


```ts
const params = [
    {
        name: "test",
        affectOldUser: true,
        createStartTime: "2024-01-07 00:00:00",
        createEndTime: "2025-01-09 00:00:00",
        effectStartTime: "2024-01-07 00:00:00",
        effectEndTime: "2025-01-09 00:00:00",
        os: [],
        channel: ["appstore", "taptap", "quick"],
        groups: [
            {
                tag: "head",
                percent: 50
            },
            {
                tag: "role",
                percent: 50
            }
        ]
    },
    {
        name: "test1",
        affectOldUser: true,
        createStartTime: "2024-01-07 00:00:00",
        createEndTime: "2025-01-09 00:00:00",
        effectStartTime: "2024-01-07 00:00:00",
        effectEndTime: "2025-01-09 00:00:00",
        os: [],
        channel: ["appstore", "taptap"],
        groups: [
            {
                tag: "head",
                percent: 50
            },
            {
                tag: "role",
                percent: 50
            }
        ]
    },
    {
        name: "test2",
        affectOldUser: true,
        createStartTime: "2024-01-08 00:00:00",
        createEndTime: "2025-01-09 00:00:00",
        effectStartTime: "2024-01-07 00:00:00",
        effectEndTime: "2025-01-09 00:00:00",
        os: [],
        channel: ["appstore", "taptap"],
        groups: [
            {
                tag: "head",
                percent: 50
            },
            {
                tag: "role",
                percent: 50
            }
        ],
    }
];
const url = "http://127.0.0.1:81/port/setAbTestCfg";
const r = await fetch(url, { 
    method: "POST", 
    body: JSON.stringify(params) 
}).then(r => r.json());
console.log("======= result ", r);
```

##### 查询ABTest配置```/port/queryAbTestCfg```

```ts
const url = "http://127.0.0.1:81/port/queryAbTestCfg";
const r = await fetch(url).then(r => r.json());
console.log("======= result ", r);
```


##### 清除用户abtest标签```/port/clearAbTestTags```

```ts
const url = "http://127.0.0.1:81/port/clearAbTestTags";
const result = await fetch(url).then(r => r.json());
console.log("============= clearAbTestTags ", result);
```


####  项目使用

##### 获取用户指定name的标签

```ts
import { getUserAbTestTags, getUserAbTestTag } from "pi_abtest/ab_test";
const result = await getUserAbTestTag("test2");
console.log("============= getUserAbTestTag ", result);
```

##### 获取用户所有标签

```ts
import { getUserAbTestTags, getUserAbTestTag } from "pi_abtest/ab_test";
const result = await getUserAbTestTags();
console.log("============= getUserAbTestTags ", result);
```


### 自动测试

### 压力测试