/**
 * ABtest
 */
import { Mgr, Tr, Item } from 'pi_ucore/db/mgr';
import { ABTest1DB, ABTestCfg1DB, RpcResult, CreateTagsParam, ABTestTagVersion1DB } from './ab_test_table.struct';
import { BISDK } from 'pi_sso_sdk/bi/bi_sdk';
import { PISYS } from "pi_sys/index";
import { tagVersionKey } from '../constant';
import { queryAbTestCfgFromDb } from '../utils';
import { IAbTestCfg } from '../interface';

declare var env;


// #[rpc=rpcServer, src=*,rpcMode=remote]
export const getUserTags = async (param: CreateTagsParam) => {
    console.log("========== getUserTags");
    const { rid, channel, os } = param;
    const abTestCfgs = await queryAbTestCfgFromDb() as IAbTestCfg[];
    const updateBI = [];
    let abtest: ABTest1DB;
    await env.dbMgr.write(async (tr: Tr) => {
        const tabName = ABTest1DB._$info.name;
        // 读取用户已有tag
        abtest = <ABTest1DB>(await tr.query([{ ware: "logfile", tab: tabName, key: rid }], 1000, false))[0].value;
        const abTestTagVersion = <ABTestTagVersion1DB>(await tr.query([{ ware: "logfile", tab: ABTestTagVersion1DB._$info.name, key: tagVersionKey }], 1000, false))[0].value;
        const currentTagVersion = abTestTagVersion ? abTestTagVersion.version : 1;
        if (abtest && abtest.tagVersion !== currentTagVersion) {
            // 版本不一致,说明有清除
            abtest = null;
        }
        let isChange = false;
        let newAbTestCfgs = [];
        if (!abtest) {
            abtest = new ABTest1DB();
            abtest.key = rid;
            abtest.tagVersion = currentTagVersion;
            abtest.value = new Map();
            newAbTestCfgs = abTestCfgs;
            isChange = true;
        } else {
            // 判断是否有删除
            const deleteNames = [];
            abtest.value.forEach((v, k) => {
                if (!abTestCfgs.find(item => item.name === k)) {
                    deleteNames.push(k);
                }
            })
            deleteNames.forEach(k => {
                isChange = true;
                abtest.value.delete(k);
            })

            // 判断是否有新增
            for (const cfg of abTestCfgs) {
                if (!abtest.value.has(cfg.name)) {
                    isChange = true;
                    if (cfg.affectOldUser) {
                        newAbTestCfgs.push(cfg);
                    } else {
                        abtest.value.set(cfg.name, "");
                    }

                }
            }

        }

        for (const cfg of newAbTestCfgs) {
            if (checkCanCreateTag(cfg, channel, os)) {
                const percents = [];
                for (const group of cfg.groups) {
                    percents.push(group.percent);
                }
                // 权重获取指定tag
                const index = randomWeightArr(percents);
                // 上报BI
                updateBI.push({ name: cfg.name, tag: cfg.groups[index].tag });
                // 添加新的ABtest
                abtest.value.set(cfg.name, cfg.groups[index].tag)
            } else {
                abtest.value.set(cfg.name, "");
            }

        }
        // 值有新增，写入数据库
        if (isChange) {
            await tr.modify([{ ware: "logfile", tab: ABTest1DB._$info.name, key: rid, value: abtest }], 1000, true);
        }
    }, 3000);
    if (!biSdk) {
        biSdk = new BISDK(PISYS.Env.get("name"));
    }
    for (const tag of updateBI) {
        const tagName = tag.name + "." + tag.tag;
        biSdk.abTest([{ uid: rid, tagName }]);
    }
    const newMap: Map<string, string> = new Map;
    abtest.value.forEach((v, k) => {
        newMap.set(k, v);
        const cfg = abTestCfgs.find(item => item.name === k) as IAbTestCfg;
        if (cfg.fixedTag) {

            newMap.set(k, cfg.fixedTag);
        } else {
            if (!checkTime(cfg.effectStartTime, cfg.effectEndTime)) {
                newMap.set(k, "");
            }
        }
    });

    return newMap;
}

/**
 * 根据权重数组随机(返回数组下标)
 * @param weight [20, 30, 50]
 */
export const randomWeightArr = (weight: number[]) => {

    const random = Math.random(), psum = [];
    let sum = 0
    for (const w of weight) {
        sum += w;		// 统计概率总和 
        psum.push(sum);
    }

    const targ = random * sum;

    let lo = 0, hi = psum.length - 1;
    while (lo != hi) {
        var mid = Math.floor((lo + hi) / 2);
        if (targ >= psum[mid]) {
            lo = mid + 1;
        } else {
            hi = mid;
        }
    }

    return lo;
};


const checkCanCreateTag = (cfg: IAbTestCfg, channel?: string, os?: string) => {
    let createTag = true;
    // 判断开启时间
    if (!checkTime(cfg.createStartTime, cfg.createEndTime)) {
        createTag = false;
    }
    if (channel && !cfg.channel.includes(channel)) {
        createTag = false;
    }
    if (os && !cfg.os.includes(os)) {
        createTag = false;
    }
    return createTag;
}

const checkTime = (startTime: string, endTime: string) => {
    const now = Date.now();
    const startTimes = new Date(startTime).getTime();
    const endTimes = new Date(endTime).getTime();
    if (startTimes > now || now > endTimes) {
        return false;
    }
    return true;
}

let biSdk: BISDK;