/**
 * ABtest
 */
import { Item } from 'pi_ucore/db/mgr';
import { ABTestCfg1DB, RpcResult, ABTestTagVersion1DB } from './ab_test_table.struct';
import { tagVersionKey } from '../constant';
import { queryAbTestCfgFromDb } from '../utils';
import { IAbTestCfg } from '../interface';

declare var env;


//#[httpRpc=/port/setAbTestCfg]
export const setAbTestCfg = async (req: any) => {
    const param = req.getData();
    const params = JSON.parse(param[""]) as IAbTestCfg[];
    console.log("=========== setAbTestCfg ", params);
    const errorMsg = checkAbTestCfg(params);
    const rpcResult = new RpcResult();
    if (errorMsg) {
        rpcResult.errCode = 1;
        rpcResult.msg = errorMsg;
        return rpcResult;
    }
    const oldCfgs = await queryAbTestCfgFromDb();
    const deleteCfgs = [];
    for (let i = 0; i < oldCfgs.length; i++) {
        const item = oldCfgs[i];
        if (!params.find(newItem => newItem.name === item.name)) {
            deleteCfgs.push(item.name);
        }
    }
    const tabName = ABTestCfg1DB._$info.name;
    await env.dbMgr.write(async tr => {
        const arr: Item[] = [];
        for (let i = 0; i < params.length; i++) {
            const item = params[i];
            console.log("=============== item ", item);
            const cfgDb = new ABTestCfg1DB();
            cfgDb.name = item.name;
            cfgDb.cfg = item;
            arr.push({
                ware: "logfile", tab: tabName, key: cfgDb.name, value: cfgDb
            })
        }
        for (let i = 0; i < deleteCfgs.length; i++) {
            arr.push({
                ware: "logfile", tab: tabName, key: deleteCfgs[i], value: null
            })
        }
        await tr.modify(arr, 1000, false);
    });
    rpcResult.errCode = 0;
    rpcResult.msg = "success";
    return rpcResult;
}


//#[httpRpc=/port/queryAbTestCfg]
export const queryAbTestCfg = async () => {
    const cfgs = await queryAbTestCfgFromDb();
    return cfgs;
}


//#[httpRpc=/port/clearAbTestTags]
export const clearAbTestTags = async () => {
    console.log("============ clearAbTestTags")
    await env.dbMgr.write(async tr => {
        let abTestTagVersion = <ABTestTagVersion1DB>(await tr.query([{ ware: "logfile", tab: ABTestTagVersion1DB._$info.name, key: tagVersionKey }], 1000, false))[0].value;
        let currentTagVersion = abTestTagVersion ? abTestTagVersion.version : 1;
        currentTagVersion++;
        abTestTagVersion = new ABTestTagVersion1DB();
        abTestTagVersion.key = tagVersionKey;
        abTestTagVersion.version = currentTagVersion;
        await tr.modify([{ ware: "logfile", tab: ABTestTagVersion1DB._$info.name, key: tagVersionKey, value: abTestTagVersion }], 1000, true);
    });
    const rpcResult = new RpcResult();
    rpcResult.errCode = 0;
    rpcResult.msg = "success";
    return rpcResult;
}



export const checkAbTestCfg = (params: IAbTestCfg[]) => {
    for (let i = 0; i < params.length; i++) {
        const item = params[i];
        let percent = 0;
        for (let j = 0; j < item.groups.length; j++) {
            percent += item.groups[j].percent;
        }
        if (percent !== 100) {
            return `${item.name} groups percent total must be 100`;
        }
    }
    return "";
}

